==> open-xchange-documents-backend_7.10.2.orig.tar.bz2: open-xchange-documents-backend-7.10.2/com.openexchange.office.filter.ooxml.docx/src/com/openexchange/office/filter/ooxml/docx/components/TcComponent.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.ooxml.docx.components;

import org.docx4j.wml.CTTcPrChange;
import org.docx4j.wml.P;
import org.docx4j.wml.Tbl;
import org.docx4j.wml.Tc;
import org.docx4j.wml.TcPr;
import org.docx4j.wml.TcPrInner;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.office.filter.api.OCKey;
import com.openexchange.office.filter.core.DLNode;
import com.openexchange.office.filter.core.component.ComponentContext;
import com.openexchange.office.filter.core.IContentAccessor;
import com.openexchange.office.filter.core.DLList;
import com.openexchange.office.filter.ooxml.OperationDocument;
import com.openexchange.office.filter.ooxml.components.Component;
import com.openexchange.office.filter.ooxml.docx.tools.Table;
import com.openexchange.office.filter.ooxml.docx.tools.Utils;

public class TcComponent extends RootComponent {

    private final int gridPosition;

    public TcComponent(ComponentContext<OperationDocument, Component> parentContext, DLNode<Object> _node, int _componentNumber, int _gridPosition) {
        super(parentContext, _node, _componentNumber);
        gridPosition = _gridPosition;
    }

    public int getGridPosition() {
        return gridPosition;
    }

    public int getNextGridPosition() {
        int gridSpan = 1;
        TcPr tcPr = ((Tc)getNode().getData()).getTcPr(false);
        if(tcPr!=null) {
            TcPrInner.GridSpan span = tcPr.getGridSpan();
            if (span!=null) {
                gridSpan = span.getVal().intValue();
                if (gridSpan==0) {
                    System.out.println("Error: empty gridSpan is zero... this results in problems with the component iterator");
                    gridSpan++; // increasing.. preventing an endless loop
                }
            }
        }
        return gridPosition + gridSpan;
    }
	@Override
	public Component getNextChildComponent(ComponentContext<OperationDocument, Component> previousChildContext, Component previousChildComponent) {
        final DLNode<Object> cellNode = getNode();
        final DLList<Object> nodeList = (DLList<Object>)((IContentAccessor)cellNode.getData()).getContent();
        final int nextComponentNumber = previousChildComponent!=null?previousChildComponent.getNextComponentNumber():0;
        DLNode<Object> childNode = previousChildContext!=null ? previousChildContext.getNode().getNext() : nodeList.getFirstNode();

        Component nextComponent = null;
        for(; nextComponent==null&&childNode!=null; childNode = childNode.getNext()) {
            final Object o = getContentModel(childNode, cellNode.getData());
            if(o instanceof P) {
                nextComponent = new ParagraphComponent(this, childNode, nextComponentNumber);
            }
            else if(o instanceof Tbl) {
                nextComponent = new TableComponent(this, childNode, nextComponentNumber);
            }
        }
        return nextComponent;
	}
	@Override
	public void applyAttrsFromJSON(JSONObject attrs) {
		if(attrs==null) {
			return;
		}
		try {
	        final Tc tc = (Tc)getObject();
	        final TcPr tcPr = tc.getTcPr(true);
	        Table.applyCellProperties((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, attrs.optJSONObject(OCKey.CELL.value()), tcPr);
	        final Object changes = attrs.opt(OCKey.CHANGES.value());
	        if(changes!=null) {
		        if(changes instanceof JSONObject) {
		        	final Object attrsInserted = ((JSONObject)changes).opt(OCKey.INSERTED.value());
		        	if(attrsInserted!=null) {
		        		if(attrsInserted instanceof JSONObject) {
                			Utils.applyTrackInfoFromJSON((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, (JSONObject)attrsInserted, tcPr.getCellIns(true));
		        		}
			        	else {
			        		tcPr.setCellIns(null);
			        	}
		        	}
		        	final Object attrsRemoved = ((JSONObject)changes).opt(OCKey.REMOVED.value());
		        	if(attrsRemoved!=null) {
	            		if(attrsRemoved instanceof JSONObject) {
	            			Utils.applyTrackInfoFromJSON((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, (JSONObject)attrsRemoved, tcPr.getCellDel(true));
	            		}
	            		else {
	            			tcPr.setCellDel(null);
	            		}
		        	}
		        	final Object attrsModified = ((JSONObject)changes).opt(OCKey.MODIFIED.value());
		        	if(attrsModified!=null) {
		        		if(attrsModified instanceof JSONObject) {
		        		    final CTTcPrChange tcPrChange = tcPr.getTcPrChange(true);
	            			Utils.applyTrackInfoFromJSON((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, (JSONObject)attrsModified, tcPrChange);
        					final Object attrsModifiedAttrs = ((JSONObject)attrsModified).opt(OCKey.ATTRS.value());
        					if(attrsModifiedAttrs!=null) {
        						if(attrsModifiedAttrs instanceof JSONObject) {
        					        Table.applyCellProperties((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, ((JSONObject)attrsModifiedAttrs).optJSONObject(OCKey.CELL.value()), tcPrChange.getTcPr(true));
        						}
        						else {
        							tcPrChange.setTcPr(null);
        						}
        					}
		        		}
		        		else {
		        			tcPr.setTcPrChange(null);
		        		}
		        	}
		        }
		        else {
		        	tcPr.setCellDel(null);
		        	tcPr.setCellIns(null);
		        	tcPr.setCellMerge(null);
		        }
	        }
		}
		catch(Exception e) {
		    //
		}
	}

	@Override
	public JSONObject createJSONAttrs(JSONObject attrs)
		throws JSONException {

        final Tc cell = (Tc)getObject();
        final TrComponent trComponent = (TrComponent)getParentComponent();
        final TableComponent tblComponent = (TableComponent)trComponent.getParentComponent();
        final Tbl tbl = (Tbl)tblComponent.getObject();
        final TcPr tcPr = cell.getTcPr(false);
        if(tcPr!=null) {
        	Table.createCellProperties(tcPr, tbl, attrs);
            if(tcPr.getCellDel(false)!=null||tcPr.getCellIns(false)!=null||tcPr.getTcPrChange(false)!=null||tcPr.getCellMerge(false)!=null) {
                final JSONObject jsonCellChanges = new JSONObject(2);
                if(tcPr.getCellDel(false)!=null) {
                	jsonCellChanges.put(OCKey.REMOVED.value(), Utils.createJSONFromTrackInfo(getOperationDocument(), tcPr.getCellDel(false)));
                }
                if(tcPr.getCellIns(false)!=null) {
                	jsonCellChanges.put(OCKey.INSERTED.value(), Utils.createJSONFromTrackInfo(getOperationDocument(), tcPr.getCellIns(false)));
                }
                if(tcPr.getTcPrChange(false)!=null) {
                	final JSONObject jsonCellModified = Utils.createJSONFromTrackInfo(getOperationDocument(), tcPr.getTcPrChange(false));
                	final JSONObject jsonCellAttrChanges = new JSONObject();
                	final TcPrInner tcPrInner = tcPr.getTcPrChange(false).getTcPr(false);
                	if(tcPrInner!=null) {
                    	Table.createCellProperties(tcPrInner, tbl, jsonCellAttrChanges);
                	}
                	if(!jsonCellAttrChanges.isEmpty()) {
                		jsonCellModified.put(OCKey.ATTRS.value(), jsonCellAttrChanges);
                	}
                	if(!jsonCellModified.isEmpty()) {
                		jsonCellChanges.put(OCKey.MODIFIED.value(), jsonCellModified);
                	}
                }
                if(tcPr.getCellMerge(false)!=null) {
                	jsonCellChanges.put(OCKey.MERGED.value(), Utils.createJSONFromTrackInfo(getOperationDocument(), tcPr.getCellMerge(false)));
                }
                if(!jsonCellChanges.isEmpty()) {
                	attrs.put(OCKey.CHANGES.value(), jsonCellChanges);
                }
            }
        }
		return attrs;
	}
}

==> open-xchange-documents-backend_7.10.3.orig.tar.bz2: open-xchange-documents-backend-7.10.3/com.openexchange.office.filter.ooxml.docx/src/com/openexchange/office/filter/ooxml/docx/components/TcComponent.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.ooxml.docx.components;

import org.docx4j.wml.CTTcPrChange;
import org.docx4j.wml.P;
import org.docx4j.wml.SdtBlock;
import org.docx4j.wml.Tbl;
import org.docx4j.wml.Tc;
import org.docx4j.wml.TcPr;
import org.docx4j.wml.TcPrInner;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.office.filter.api.OCKey;
import com.openexchange.office.filter.core.DLNode;
import com.openexchange.office.filter.core.component.Component;
import com.openexchange.office.filter.core.component.ComponentContext;
import com.openexchange.office.filter.core.IContentAccessor;
import com.openexchange.office.filter.core.DLList;
import com.openexchange.office.filter.ooxml.OfficeOpenXMLOperationDocument;
import com.openexchange.office.filter.ooxml.docx.tools.Table;
import com.openexchange.office.filter.ooxml.docx.tools.Utils;

public class TcComponent extends RootComponent {

    private final int gridPosition;

    public TcComponent(ComponentContext<OfficeOpenXMLOperationDocument> parentContext, DLNode<Object> _node, int _componentNumber, int _gridPosition) {
        super(parentContext, _node, _componentNumber);
        gridPosition = _gridPosition;
    }

    public int getGridPosition() {
        return gridPosition;
    }

    public int getNextGridPosition() {
        int gridSpan = 1;
        TcPr tcPr = ((Tc)getNode().getData()).getTcPr(false);
        if(tcPr!=null) {
            TcPrInner.GridSpan span = tcPr.getGridSpan();
            if (span!=null) {
                gridSpan = span.getVal().intValue();
                if (gridSpan==0) {
                    System.out.println("Error: empty gridSpan is zero... this results in problems with the component iterator");
                    gridSpan++; // increasing.. preventing an endless loop
                }
            }
        }
        return gridPosition + gridSpan;
    }

    @Override
	public Component<OfficeOpenXMLOperationDocument> getNextChildComponent(ComponentContext<OfficeOpenXMLOperationDocument> previousChildContext, Component<OfficeOpenXMLOperationDocument> previousChildComponent) {
        final DLNode<Object> cellNode = getNode();
        final DLList<Object> nodeList = (DLList<Object>)((IContentAccessor)cellNode.getData()).getContent();
        final int nextComponentNumber = previousChildComponent!=null?previousChildComponent.getNextComponentNumber():0;
        DLNode<Object> childNode = previousChildContext!=null ? previousChildContext.getNode().getNext() : nodeList.getFirstNode();

        Component<OfficeOpenXMLOperationDocument> nextComponent = null;
        for(; nextComponent==null&&childNode!=null; childNode = childNode.getNext()) {
            final Object o = getContentModel(childNode, cellNode.getData());
            if(o instanceof P) {
                nextComponent = new ParagraphComponent(this, childNode, nextComponentNumber);
            }
            else if(o instanceof Tbl) {
                nextComponent = new TableComponent(this, childNode, nextComponentNumber);
            }
            else if(o instanceof SdtBlock) {
                final SdtRootContext sdtRootContext = new SdtRootContext(this, childNode);
                nextComponent = sdtRootContext.getNextChildComponent(null, previousChildComponent);
            }
        }
        return nextComponent;
	}

	@Override
	public void applyAttrsFromJSON(JSONObject attrs) {
		if(attrs==null) {
			return;
		}
		try {
	        final Tc tc = (Tc)getObject();
	        final TcPr tcPr = tc.getTcPr(true);
	        Table.applyCellProperties((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, attrs.optJSONObject(OCKey.CELL.value()), tcPr);
	        final Object changes = attrs.opt(OCKey.CHANGES.value());
	        if(changes!=null) {
		        if(changes instanceof JSONObject) {
		        	final Object attrsInserted = ((JSONObject)changes).opt(OCKey.INSERTED.value());
		        	if(attrsInserted!=null) {
		        		if(attrsInserted instanceof JSONObject) {
                			Utils.applyTrackInfoFromJSON((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, (JSONObject)attrsInserted, tcPr.getCellIns(true));
		        		}
			        	else {
			        		tcPr.setCellIns(null);
			        	}
		        	}
		        	final Object attrsRemoved = ((JSONObject)changes).opt(OCKey.REMOVED.value());
		        	if(attrsRemoved!=null) {
	            		if(attrsRemoved instanceof JSONObject) {
	            			Utils.applyTrackInfoFromJSON((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, (JSONObject)attrsRemoved, tcPr.getCellDel(true));
	            		}
	            		else {
	            			tcPr.setCellDel(null);
	            		}
		        	}
		        	final Object attrsModified = ((JSONObject)changes).opt(OCKey.MODIFIED.value());
		        	if(attrsModified!=null) {
		        		if(attrsModified instanceof JSONObject) {
		        		    final CTTcPrChange tcPrChange = tcPr.getTcPrChange(true);
	            			Utils.applyTrackInfoFromJSON((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, (JSONObject)attrsModified, tcPrChange);
        					final Object attrsModifiedAttrs = ((JSONObject)attrsModified).opt(OCKey.ATTRS.value());
        					if(attrsModifiedAttrs!=null) {
        						if(attrsModifiedAttrs instanceof JSONObject) {
        					        Table.applyCellProperties((com.openexchange.office.filter.ooxml.docx.DocxOperationDocument)operationDocument, ((JSONObject)attrsModifiedAttrs).optJSONObject(OCKey.CELL.value()), tcPrChange.getTcPr(true));
        						}
        						else {
        							tcPrChange.setTcPr(null);
        						}
        					}
		        		}
		        		else {
		        			tcPr.setTcPrChange(null);
		        		}
		        	}
		        }
		        else {
		        	tcPr.setCellDel(null);
		        	tcPr.setCellIns(null);
		        	tcPr.setCellMerge(null);
		        }
	        }
		}
		catch(Exception e) {
		    //
		}
	}

	@Override
	public JSONObject createJSONAttrs(JSONObject attrs)
		throws JSONException {

        final Tc cell = (Tc)getObject();
        final TrComponent trComponent = (TrComponent)getParentComponent();
        final TableComponent tblComponent = (TableComponent)trComponent.getParentComponent();
        final Tbl tbl = (Tbl)tblComponent.getObject();
        final TcPr tcPr = cell.getTcPr(false);
        if(tcPr!=null) {
        	Table.createCellProperties(tcPr, tbl, attrs);
            if(tcPr.getCellDel(false)!=null||tcPr.getCellIns(false)!=null||tcPr.getTcPrChange(false)!=null||tcPr.getCellMerge(false)!=null) {
                final JSONObject jsonCellChanges = new JSONObject(2);
                if(tcPr.getCellDel(false)!=null) {
                	jsonCellChanges.put(OCKey.REMOVED.value(), Utils.createJSONFromTrackInfo(getOperationDocument(), tcPr.getCellDel(false)));
                }
                if(tcPr.getCellIns(false)!=null) {
                	jsonCellChanges.put(OCKey.INSERTED.value(), Utils.createJSONFromTrackInfo(getOperationDocument(), tcPr.getCellIns(false)));
                }
                if(tcPr.getTcPrChange(false)!=null) {
                	final JSONObject jsonCellModified = Utils.createJSONFromTrackInfo(getOperationDocument(), tcPr.getTcPrChange(false));
                	final JSONObject jsonCellAttrChanges = new JSONObject();
                	final TcPrInner tcPrInner = tcPr.getTcPrChange(false).getTcPr(false);
                	if(tcPrInner!=null) {
                    	Table.createCellProperties(tcPrInner, tbl, jsonCellAttrChanges);
                	}
                	if(!jsonCellAttrChanges.isEmpty()) {
                		jsonCellModified.put(OCKey.ATTRS.value(), jsonCellAttrChanges);
                	}
                	if(!jsonCellModified.isEmpty()) {
                		jsonCellChanges.put(OCKey.MODIFIED.value(), jsonCellModified);
                	}
                }
                if(tcPr.getCellMerge(false)!=null) {
                	jsonCellChanges.put(OCKey.MERGED.value(), Utils.createJSONFromTrackInfo(getOperationDocument(), tcPr.getCellMerge(false)));
                }
                if(!jsonCellChanges.isEmpty()) {
                	attrs.put(OCKey.CHANGES.value(), jsonCellChanges);
                }
            }
        }
		return attrs;
	}
}
